@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "base/sqlstring.h"
 #include "base/log.h"
 #include "base/threading.h"
@@ -95,6 +93,7 @@ static std::string query_type_to_help_topic[] = {
   "create database",      // QtCreateDatabase
   "create event",         // QtCreateEvent
   "create view",          // QtCreateView
+  "create procedure",     // QtCreateRoutine
   "create procedure",     // QtCreateProcedure
   "create function",      // QtCreateFunction
   "create function udf",  // QtCreateUdf
@@ -161,7 +160,7 @@ static std::string query_type_to_help_topic[] = {
   "analyze table",        // QtAnalyzeTable
   "check table",          // QtCheckTable
   "checksum table",       // QtChecksumTable
-  "optimze table",        // QtOptimizeTable
+  "optimize table",       // QtOptimizeTable
   "repair table",         // QtRepairTable
   "backup table",         // QtBackUpTable
   "restore table",        // QtRestoreTable
@@ -481,7 +480,7 @@ bool DbSqlEditorContextHelp::get_help_text(const SqlEditorForm::Ref &form, const
     }
     catch (...)
     {
-      log_debug2("Exception caught while looking up topic\n");
+      log_debug2("Exception caught while looking up help text\n");
     }
   }
   return false;
@@ -493,7 +492,7 @@ bool DbSqlEditorContextHelp::get_help_text(const SqlEditorForm::Ref &form, const
  * Determines a help topic from the given query at the given position.
  */
 std::string DbSqlEditorContextHelp::find_help_topic_from_position(const SqlEditorForm::Ref &form,
-  const std::string &query, std::pair<int, int> caret)
+  const std::string &query, std::pair<ssize_t, ssize_t> caret)
 {
   log_debug2("Finding help topic\n");
   
@@ -513,17 +512,17 @@ std::string DbSqlEditorContextHelp::find_help_topic_from_position(const SqlEdito
   // There are a few cases where we need a parser to decide. That might however fail due to
   // syntax errors. So we check first these special cases if we can solve them by
   // parsing. If not we continue with our normal strategy.
-  MySQLRecognizer recognizer(query.c_str(), query.length(), true, form->server_version(),
-    form->sql_mode(), form->valid_charsets());
+  MySQLRecognizer recognizer(form->server_version(), form->sql_mode(), form->valid_charsets());
+  recognizer.parse(query.c_str(), query.length(), true, QtUnknown);
   MySQLRecognizerTreeWalker walker = recognizer.tree_walker();
-  bool found_token = walker.advance_to_position(caret.second, caret.first);
+  bool found_token = walker.advance_to_position((int)caret.second, (int)caret.first);
   if (found_token && recognizer.has_errors())
   {
     // We can only assume success if the first error is after our position. Otherwise
     // we cannot predict what's in the syntax tree.
     MySQLParserErrorInfo error = recognizer.error_info().front();
     found_token = ((int)error.line > caret.second ||
-      (int)error.line == caret.second && (int)error.offset > caret.first);
+      (int)error.line == caret.second && (int)error.charOffset > caret.first);
   }
 
   if (found_token)
@@ -629,11 +628,9 @@ std::string DbSqlEditorContextHelp::find_help_topic_from_position(const SqlEdito
 //--------------------------------------------------------------------------------------------------
 
 /**
- * A stripped down variant as used in the scanner to determine if the given type is an operator
- * for which no help topic exists. We don't do topic lookup though, but use a fixed list as this
- * won't change.
+ * Checks for tokens we know there is no help topic for.
  */
-bool is_operator_without_topic(unsigned type)
+bool is_token_without_topic(unsigned type)
 {
   switch (type)
   {
@@ -646,6 +643,9 @@ bool is_operator_without_topic(unsigned type)
     case AT_SIGN_SYMBOL:
     case AT_AT_SIGN_SYMBOL:
     case PARAM_MARKER:
+    case SINGLE_QUOTED_TEXT:
+    case BACK_TICK_QUOTED_ID:
+    case DOUBLE_QUOTED_TEXT:
       return true;
 
   default:
@@ -660,10 +660,10 @@ bool is_operator_without_topic(unsigned type)
  * This includes all single word topics, so they don't need to be covered later again.
  */
 std::string DbSqlEditorContextHelp::topic_from_position(const SqlEditorForm::Ref &form,
-  const std::string &query, std::pair<int, int> caret)
+  const std::string &query, std::pair<ssize_t, ssize_t> caret)
 {
-  // Don't log the entire query. That can really take a moment with large queries.
-  log_debug2("Trying to get help topic from a position\n");
+  log_debug2("Trying to get help topic at position <%li, %li>, from query: %s...\n", (long)caret.first,
+    (long)caret.second, query.substr(0, 300).c_str());
   
   // First collect all tokens up to the caret position.
   MySQLScanner scanner(query.c_str(), query.length(), true, form->server_version(), form->sql_mode(),
@@ -686,7 +686,7 @@ std::string DbSqlEditorContextHelp::topic_from_position(const SqlEditorForm::Ref
   size_t index = tokens.size() - 1;
   while (index > 0)
   {
-    if (token.channel != 0 || scanner.is_number(token.type) || is_operator_without_topic(token.type))
+    if (token.channel != 0 || scanner.is_number(token.type) || is_token_without_topic(token.type))
     {
       token = tokens[--index];
       continue;
@@ -1041,6 +1041,19 @@ std::string DbSqlEditorContextHelp::topic_from_position(const SqlEditorForm::Ref
     }
     break;
 
+  case SESSION_SYMBOL:
+    token = get_previous_real_token(tokens, index);
+    switch (token.type)
+    {
+    case SHOW_SYMBOL:
+      topic = "show";
+      break;
+
+    case SET_SYMBOL:
+      topic = "set";
+      break;
+    }
+    break;
   }
 
   return topic; // Could be empty. This is checked at the caller level.
