@@ -17,24 +17,24 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #ifndef _WIN32
 #include <pcre.h>
 #include <stdio.h>
 #endif
 
+#include "base/string_utilities.h"
+#include "base/util_functions.h"
+#include "base/log.h"
+
 #include <boost/scoped_array.hpp>
 
 #include "db_object_helpers.h"
-#include <grtpp_undo_manager.h>
-#include <grtpp_util.h>
+#include "grtpp_undo_manager.h"
+#include "grtpp_util.h"
 #include "grt/common.h"
 #include "grt/parse_utils.h"
 #include "grts/structs.workbench.physical.h"
-#include "base/string_utilities.h"
 #include "grtdb/db_helpers.h"
-#include "base/log.h"
 
 DEFAULT_LOG_DOMAIN("dbhelpers");
 
@@ -316,7 +316,7 @@ db_TableRef TableHelper::create_associative_table(const db_SchemaRef &schema,
   first_fk->index(index);
   atable->indices().insert(index);
 
-  atable->createDate(grt::StringRef(fmttime(0, DATETIME_FMT)));
+  atable->createDate(grt::StringRef(base::fmttime(0, DATETIME_FMT)));
   atable->lastChangeDate(atable->createDate());
   
   undo.end("Create Associative Table");
@@ -595,7 +595,7 @@ void TableHelper::update_foreign_key_index(const db_ForeignKeyRef &fk)
     // make sure that the columns in the index match the ones in the FK
 
     // remove columns that are gone from the FK
-    for (int i = index->columns().count()-1; i >= 0; --i)
+    for (ssize_t i = index->columns().count() - 1; i >= 0; --i)
     {
       if (fk->columns().get_index(index->columns()[i]) == grt::BaseListRef::npos)
         index->columns().remove(i);
@@ -694,7 +694,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   std::string column_name_format;
   std::string scolumn_name;
   std::string dcolumn_name;
-  int max_identifier_length= rdbms->maximumIdentifierLength();
+  size_t max_identifier_length= rdbms->maximumIdentifierLength();
 
   // check if there is a PK
   if (!pk.is_valid() || pk->columns().count() == 0)
@@ -759,7 +759,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   
   new_fk->name(SchemaHelper::get_unique_foreign_key_name(db_SchemaRef::cast_from(table->owner()), 
                                                          name_format, 
-                                                         max_identifier_length));
+                                                         (int)max_identifier_length));
   new_fk->oldName(new_fk->name());
   
   new_fk->owner(table); // set the owner last
@@ -802,7 +802,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   std::string name_format;
   std::string scolumn_name;
   std::string dcolumn_name;
-  int max_identifier_length= rdbms->maximumIdentifierLength();
+  size_t max_identifier_length= rdbms->maximumIdentifierLength();
   bool ref_mandatory;
 
   grt::AutoUndo undo(grt, table->is_global());
@@ -848,7 +848,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   
   new_fk->name(SchemaHelper::get_unique_foreign_key_name(db_SchemaRef::cast_from(table->owner()), 
                                                          name_format,
-                                                         max_identifier_length));
+                                                         (int)max_identifier_length));
   new_fk->oldName(new_fk->name());
   
   new_fk->referencedMandatory(ref_mandatory?1:0);
@@ -942,7 +942,7 @@ db_mysql_StorageEngineRef TableHelper::get_engine_by_name(grt::GRT *grt, const s
 }
 
 /**
- * Returns a pointer to the position of the first line break or the first character after the
+ * Returns a pointer to the position of the first empty line (NOT a line break!) or the first character after the
  * maximum count, whichever comes first. If no line break was found and the string is shorter
  * than the maximum length the result points to the terminating 0.
  */
@@ -951,7 +951,16 @@ static void split_comment(const std::string &comment, size_t db_comment_len,
 {
   size_t res;
   // XXX: check for Unicode line breaks! especially asian languages don't use the ANSI new line.
-  gchar* pointer_to_linebreak = g_utf8_strchr(comment.c_str(), comment.size(), '\n');
+  const gchar* pointer_to_linebreak = NULL;
+
+  { // find 1st occurrence of 2 consecutive newlines
+    std::string::size_type pos = comment.find("\n\n");
+    if (pos == std::string::npos)
+      pos = comment.find("\r\n\r\n");
+
+    if (pos != std::string::npos)
+      pointer_to_linebreak = comment.c_str() + pos;
+  }
 
   // We need the number of characters which the string part includes, so convert to a char count.
   if (pointer_to_linebreak != NULL)
@@ -962,7 +971,7 @@ static void split_comment(const std::string &comment, size_t db_comment_len,
   // don't break in the middle of a utf8 sequence
   if (res > db_comment_len)
   {
-    if (g_utf8_get_char_validated(comment.c_str() + db_comment_len, res - db_comment_len) == (gunichar)-1)
+    if (g_utf8_get_char_validated(comment.c_str() + db_comment_len, (gssize)(res - db_comment_len)) == (gunichar)-1)
       res = g_utf8_pointer_to_offset(comment.c_str(), g_utf8_find_prev_char(comment.c_str(), comment.c_str() + db_comment_len));
     else
       res = db_comment_len;
@@ -981,7 +990,10 @@ static void split_comment(const std::string &comment, size_t db_comment_len,
 std::string TableHelper::get_sync_comment(const std::string &comment, const size_t max_len)
 {
   std::string ret;
-  split_comment(comment, max_len, &ret, NULL);
+  if (comment.size() > max_len)
+    split_comment(comment, max_len, &ret, NULL);
+  else
+    ret = comment;
   return ret;
 };
 
@@ -1244,7 +1256,7 @@ static int pcre_compile_exec(const char *pattern, const char *str, int *patres,
   if (!patre)
     throw std::logic_error("error compiling regex "+std::string(errptr));
 
-  c= pcre_exec(patre, NULL, str, strlen(str), 0, 0, patres, patresnum);
+  c= pcre_exec(patre, NULL, str, (int)strlen(str), 0, 0, patres, patresnum);
   pcre_free(patre);
 
   return c;
@@ -1413,13 +1425,13 @@ static bool parseTypeDefinition(const std::string &type,
       if (*simpleType->numericPrecision() != bec::EMPTY_TYPE_PRECISION)
       {
         if (!param1.empty())
-          precision= atoi(param1.c_str());
+          precision= base::atoi<int>(param1, 0);
         
         // get scale
         if (*simpleType->numericScale() != bec::EMPTY_TYPE_SCALE)
         {
           if (!param2.empty())
-            scale= atoi(param2.c_str());
+            scale= base::atoi<int>(param2, 0);
         }
       }
       // get length
@@ -1427,7 +1439,7 @@ static bool parseTypeDefinition(const std::string &type,
                || simpleType->characterOctetLength()   != bec::EMPTY_TYPE_OCTET_LENGTH)
       {
         if (!param1.empty())
-          length= atoi(param1.c_str());
+          length= base::atoi<int>(param1, 0);
       }
       else if (*simpleType->parameterFormatType() == 10)
         explicitParams= param1;
@@ -1551,18 +1563,25 @@ std::string bec::get_default_collation_for_charset(const db_TableRef &table, con
 
 std::string bec::TableHelper::generate_comment_text(const std::string& comment_text, size_t comment_lenght)
 {
-  std::string comment, leftover;
-
-  split_comment(comment_text, comment_lenght, &comment, &leftover);
-
-  if (!comment.empty())
-      comment = "'"+base::escape_sql_string(comment)+"'";
-  if (!leftover.empty())
+  if (comment_text.size() > comment_lenght)
   {
-    base::replace(leftover, "*/", "*\\/");
-    comment.append(" /* comment truncated */ /*")
+    std::string comment, leftover;
+
+    split_comment(comment_text, comment_lenght, &comment, &leftover);
+
+    if (!comment.empty())
+      comment = "'"+base::escape_sql_string(comment)+"'";
+    if (!leftover.empty())
+    {
+      base::replace(leftover, "*/", "*\\/");
+      comment.append(" /* comment truncated */ /*")
       .append(leftover)
       .append("*/");
+    }
+    return comment;
   }
-  return comment; 
-}
\ No newline at end of file
+  else if (!comment_text.empty())
+    return "'"+base::escape_sql_string(comment_text)+"'";
+
+  return "";
+}
