@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include <sqlite/execute.hpp>
 #include <sqlite/query.hpp>
 #include <sqlite/database_exception.hpp>
@@ -95,11 +93,11 @@ AutoCompleteCache::AutoCompleteCache(const std::string &connection_id,
 
 void AutoCompleteCache::shutdown()
 {
-  base::MutexLock sd_lock(_shutdown_mutex);
+  base::RecMutexLock sd_lock(_shutdown_mutex);
   _shutdown = true;
 
   {
-    base::MutexLock lock(_pending_mutex);
+    base::RecMutexLock lock(_pending_mutex);
     _pending_refresh_schema.clear();
     _feedback = NULL;
   }
@@ -132,8 +130,13 @@ std::vector<std::string>  AutoCompleteCache::get_matching_schema_names(const std
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::MutexLock sd_lock(_shutdown_mutex);
-    base::MutexLock lock(_sqconn_mutex);
+    base::RecMutexTryLock sd_lock(_shutdown_mutex);
+    if (!sd_lock.locked())
+    {
+      log_debug3("Can't lock shutdown_mutex in get_machging_schema_names.\n");
+      return std::vector<std::string>();
+    }
+    base::RecMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM schemas WHERE name LIKE ? ESCAPE '\\'");
     q.bind(1, base::escape_sql_string(prefix, true) + "%");
     if (q.emit())
@@ -169,8 +172,13 @@ std::vector<std::string> AutoCompleteCache::get_matching_table_names(const std::
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::MutexLock sd_lock(_shutdown_mutex);
-    base::MutexLock lock(_sqconn_mutex);
+    base::RecMutexTryLock sd_lock(_shutdown_mutex);
+    if (!sd_lock.locked())
+    {
+      log_debug3("Can't lock shutdown_mutex in get_matching_table_names.\n");
+      return std::vector<std::string>();
+    }
+    base::RecMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM tables WHERE schema LIKE ? ESCAPE '\\' AND name LIKE ? ESCAPE '\\'");
     q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
     q.bind(2, base::escape_sql_string(prefix, true) + "%");
@@ -207,8 +215,13 @@ std::vector<std::string> AutoCompleteCache::get_matching_column_names(const std:
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::MutexLock sd_lock(_shutdown_mutex);
-    base::MutexLock lock(_sqconn_mutex);
+    base::RecMutexTryLock sd_lock(_shutdown_mutex);
+    if (!sd_lock.locked())
+    {
+      log_debug3("Can't lock shutdown_mutex in get_matching_column_names.\n");
+      return std::vector<std::string>();
+    }
+    base::RecMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM columns WHERE schema LIKE ? ESCAPE '\\' "
       "AND tabl LIKE ? ESCAPE '\\' AND name LIKE ? ESCAPE '\\'");
     q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
@@ -249,8 +262,13 @@ std::vector<std::string> AutoCompleteCache::get_matching_procedure_names(const s
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::MutexLock sd_lock(_shutdown_mutex);
-    base::MutexLock lock(_sqconn_mutex);
+    base::RecMutexTryLock sd_lock(_shutdown_mutex);
+    if (!sd_lock.locked())
+    {
+      log_debug3("Can't lock shutdown_mutex in get_matching_procedure_names.\n");
+      return std::vector<std::string>();
+    }
+    base::RecMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM routines WHERE schema LIKE ? ESCAPE '\\' "
       "AND name LIKE ? ESCAPE '\\' AND is_function=0");
     q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
@@ -289,8 +307,13 @@ std::vector<std::string> AutoCompleteCache::get_matching_function_names(const st
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::MutexLock sd_lock(_shutdown_mutex);
-    base::MutexLock lock(_sqconn_mutex);
+    base::RecMutexTryLock sd_lock(_shutdown_mutex);
+    if (!sd_lock.locked())
+    {
+      log_debug3("Can't lock shutdown_mutex in get_matching_function_names.\n");
+      return std::vector<std::string>();
+    }
+    base::RecMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT name FROM routines WHERE schema LIKE ? ESCAPE '\\' "
       "AND name LIKE ? ESCAPE '\\' AND is_function=1");
     q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
@@ -329,8 +352,13 @@ bool AutoCompleteCache::refresh_schema_cache_if_needed(const std::string &schema
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::MutexLock sd_lock(_shutdown_mutex);
-    base::MutexLock lock(_sqconn_mutex);
+    base::RecMutexTryLock sd_lock(_shutdown_mutex);
+    if (!sd_lock.locked())
+    {
+      log_debug3("Can't lock shutdown_mutex in refresh_schema_cache_if_needed.\n");
+      return false;
+    }
+    base::RecMutexLock lock(_sqconn_mutex);
     sqlite::query q(*_sqconn, "SELECT last_refresh FROM schemas WHERE name LIKE ? ESCAPE '\\' ");
     q.bind(1, schema.size() == 0 ? "%" : base::escape_sql_string(schema, true));
     if (q.emit())
@@ -424,7 +452,6 @@ void AutoCompleteCache::refresh_cache_thread()
 
 
   // signal the main thread that the thread is (about to be) gone
-  _refresh_thread = NULL;
   _cache_working.post();
 
   if (_feedback && !_shutdown)
@@ -464,7 +491,7 @@ void AutoCompleteCache::refresh_schemas_w()
         {
           schemas.push_back(rs->getString(1));
         }
-        log_debug2("Found %li schemas.\n", schemas.size()); 
+        log_debug2("Found %li schemas.\n", (long)schemas.size()); 
       }
       else
         log_debug2("No schema found.\n");
@@ -495,7 +522,7 @@ void AutoCompleteCache::refresh_tables_w(const std::string &schema)
           // automatically trigger fetch of columns for this table
           add_pending_refresh(schema + "\n" + table);
         }
-        log_debug2("updating %li tables...\n", tables.size());
+        log_debug2("updating %li tables...\n", (long)tables.size());
       }
       else
         log_debug2("no tables for %s\n", schema.c_str());
@@ -637,7 +664,7 @@ void AutoCompleteCache::init_db()
 
 bool AutoCompleteCache::is_schema_list_fetch_done()
 {
-  base::MutexLock lock(_sqconn_mutex);
+  base::RecMutexLock lock(_sqconn_mutex);
   sqlite::query q(*_sqconn, "select * from schemas");
   if (q.emit())
     return true;
@@ -647,7 +674,7 @@ bool AutoCompleteCache::is_schema_list_fetch_done()
 
 bool AutoCompleteCache::is_schema_tables_fetch_done(const std::string &schema)
 {
-  base::MutexLock lock(_sqconn_mutex);
+  base::RecMutexLock lock(_sqconn_mutex);
   sqlite::query q(*_sqconn, "select * from tables where schema = ?");
   q.bind(1, schema);
   if (q.emit())
@@ -657,7 +684,7 @@ bool AutoCompleteCache::is_schema_tables_fetch_done(const std::string &schema)
 
 bool AutoCompleteCache::is_schema_table_columns_fetch_done(const std::string &schema, const std::string &table)
 {
-  base::MutexLock lock(_sqconn_mutex);
+  base::RecMutexLock lock(_sqconn_mutex);
   sqlite::query q(*_sqconn, "select * from columns where schema = ? and tabl = ?");
   q.bind(1, schema);
   q.bind(2, table);
@@ -668,7 +695,7 @@ bool AutoCompleteCache::is_schema_table_columns_fetch_done(const std::string &sc
 
 bool AutoCompleteCache::is_schema_routines_fetch_done(const std::string &schema)
 {
-  base::MutexLock lock(_sqconn_mutex);
+  base::RecMutexLock lock(_sqconn_mutex);
   sqlite::query q(*_sqconn, "select * from routines where schema = ?");
   q.bind(1, schema);
   if (q.emit())
@@ -704,8 +731,14 @@ void AutoCompleteCache::update_schemas(const std::vector<std::string> &schemas)
     if (!_shutdown)
     {
       // Ensures shutdown is not done while processing
-      base::MutexLock sd_lock(_shutdown_mutex);
-      base::MutexLock lock(_sqconn_mutex);
+      base::RecMutexTryLock sd_lock(_shutdown_mutex);
+      if (!sd_lock.locked())
+      {
+        log_debug3("Can't lock shutdown_mutex in update_schemas.\n");
+        return;
+      }
+
+      base::RecMutexLock lock(_sqconn_mutex);
  
       std::map<std::string, int> old_schema_update_times;
       {
@@ -768,8 +801,14 @@ void AutoCompleteCache::update_schema_tables(const std::string &schema,
     if (!_shutdown)
     {
       // Ensures shutdown is not done while processing
-      base::MutexLock sd_lock(_shutdown_mutex);
-      base::MutexLock lock(_sqconn_mutex);
+      base::RecMutexTryLock sd_lock(_shutdown_mutex);
+      if (!sd_lock.locked())
+      {
+        log_debug3("Can't lock shutdown_mutex in update_schemas.\n");
+        return;
+      }
+
+      base::RecMutexLock lock(_sqconn_mutex);
  
       touch_schema_record(schema);
     
@@ -816,8 +855,14 @@ void AutoCompleteCache::update_schema_routines(const std::string &schema,
     if (!_shutdown)
     {
       // Ensures shutdown is not done while processing
-      base::MutexLock sd_lock(_shutdown_mutex);
-      base::MutexLock lock(_sqconn_mutex);
+      base::RecMutexTryLock sd_lock(_shutdown_mutex);
+      if (!sd_lock.locked())
+      {
+        log_debug3("Can't lock shutdown_mutex in update_schema_routines.\n");
+        return;
+      }
+
+      base::RecMutexLock lock(_sqconn_mutex);
     
       touch_schema_record(schema);
 
@@ -865,8 +910,14 @@ void AutoCompleteCache::update_table_columns(const std::string &schema, const st
     if (!_shutdown)
     {
       // Ensures shutdown is not done while processing
-      base::MutexLock sd_lock(_shutdown_mutex);
-      base::MutexLock lock(_sqconn_mutex);
+      base::RecMutexTryLock sd_lock(_shutdown_mutex);
+      if (!sd_lock.locked())
+      {
+        log_debug3("Can't lock shutdown_mutex in update_table_columns.\n");
+        return;
+      }
+
+      base::RecMutexLock lock(_sqconn_mutex);
       try
       {
         sqlide::Sqlite_transaction_guarder  trans(_sqconn, false); //will be committed when will go out of the scope
@@ -911,8 +962,13 @@ void AutoCompleteCache::add_pending_refresh(const std::string& task)
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::MutexLock sd_lock(_shutdown_mutex);
-    base::MutexLock lock(_pending_mutex);
+    base::RecMutexTryLock sd_lock(_shutdown_mutex);
+    if (!sd_lock.locked())
+    {
+      log_debug3("Can't lock shutdown_mutex in add_pending_refresh.\n");
+      return;
+    }
+    base::RecMutexLock lock(_pending_mutex);
     if ((!task.empty() || !_schema_list_fetched) && std::find(_pending_refresh_schema.begin(), _pending_refresh_schema.end(), task)
         == _pending_refresh_schema.end())
     {
@@ -932,8 +988,13 @@ bool AutoCompleteCache::get_pending_refresh(std::string &task)
   if (!_shutdown)
   {
     // Ensures shutdown is not done while processing
-    base::MutexLock sd_lock(_shutdown_mutex);
-    base::MutexLock lock(_pending_mutex);
+    base::RecMutexTryLock sd_lock(_shutdown_mutex);
+    if (!sd_lock.locked())
+    {
+      log_debug3("Can't lock shutdown_mutex in get_pending_refresh.\n");
+      return ret_val;
+    }
+    base::RecMutexLock lock(_pending_mutex);
 
     if (!_pending_refresh_schema.empty())
     {
@@ -953,7 +1014,12 @@ void AutoCompleteCache::create_worker_thread()
   if(!_cache_working.try_wait()) // If there is already working thread, just do nothing and exit.
     return;
 
-  _refresh_thread = NULL;
+  if (_refresh_thread != NULL)
+  {
+    g_thread_join(_refresh_thread);
+    _refresh_thread = NULL;
+  }
+
 //  if (!_refresh_thread)
 //  {
     if (!_shutdown)
