@@ -39,11 +39,13 @@ inline void init_gdk_thread_callbacks()
 
 //==============================================================================
 
+extern  void lf_record_grid_init();
+
 int main(int argc, char **argv)
 {
 
   // disable gnome keyring if it's not running
-  if (!getenv("GNOME_KEYRING_CONTROL") && !getenv("GNOME_KEYRING_SOCKET"))
+  if (!getenv("GNOME_KEYRING_CONTROL") && !getenv("GNOME_KEYRING_SOCKET") && !getenv("XDG_RUNTIME_DIR"))
   {
     g_message("Gnome keyring daemon seems to not be available. Stored passwords will be lost once quit");
     setenv("WB_NO_GNOME_KEYRING", "1", 1);
@@ -70,10 +72,12 @@ int main(int argc, char **argv)
   base::threading_init();
   gdk_threads_init();
   // process cmdline options
+  std::string user_data_dir = std::string(g_get_home_dir()).append("/.mysql/workbench");
+  base::Logger log(user_data_dir, getenv("MWB_LOG_TO_STDERR")!=NULL);
+
   wb::WBOptions wboptions;
+  wboptions.user_data_dir = user_data_dir;
 
-  wboptions.user_data_dir = std::string(g_get_home_dir()).append("/.mysql/workbench");
-  base::Logger log(wboptions.user_data_dir, getenv("MWB_LOG_TO_STDERR")!=NULL);
 
   wboptions.basedir = getenv("MWB_DATA_DIR");
   wboptions.plugin_search_path = getenv("MWB_PLUGIN_DIR");
@@ -91,6 +95,9 @@ int main(int argc, char **argv)
 
   mforms::gtk::init(getenv("WB_FORCE_SYSTEM_COLORS") != NULL);
   mforms::gtk::WizardImpl::set_icon_path(wboptions.basedir+"/images");
+  {
+    lf_record_grid_init();
+  }
 
   Gtk::RC::add_default_file(wboptions.basedir+"/workbench.rc");
 
