@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -126,6 +126,19 @@ public:
   return changed;
 }
 
+
+- (void)becomeKeyWindow
+{
+  mforms::Form::main_form()->activated();
+  [super becomeKeyWindow];
+}
+
+- (void)resignKeyWindow
+{
+  [super resignKeyWindow];
+  mforms::Form::main_form()->deactivated();
+}
+
 @end
 
 
@@ -276,7 +289,6 @@ void setup_mforms_app(WBMainWindow *mwin);
   const char* RefreshTypeStr[] = {
     "RefreshNeeded",
     "RefreshNothing",
-    "RefreshSchemaList",
     "RefreshSchemaNoReload",
     "RefreshNewView",
     "RefreshSelection",
@@ -367,14 +379,6 @@ void setup_mforms_app(WBMainWindow *mwin);
       break;
     }
       
-    case wb::RefreshSchemaList:
-    {
-      id panel = [self selectedTopPanel];
-      if ([panel respondsToSelector:@selector(refreshCatalogTree)])
-        [panel refreshCatalogTree];
-      break;
-    }
-
     case wb::RefreshSchemaNoReload:
       break;
 
@@ -443,9 +447,8 @@ void setup_mforms_app(WBMainWindow *mwin);
                           name:(const char*)name
 {
   wb::ModelDiagramForm *dform= _wbui->get_wb()->get_model_context()->get_diagram_form_for_diagram_id(oid);
-  WBModelDiagramPanel *panel= [[WBModelDiagramPanel alloc] initWithId:[NSString stringWithUTF8String:oid]
-                                                               formBE:dform
-                                              sharedSidebarController:[_physicalOverview sidebarController]];
+  WBModelDiagramPanel *panel= [[WBModelDiagramPanel alloc] initWithId: [NSString stringWithUTF8String:oid]
+                                                               formBE: dform];
 
   NSTabViewItem* item= [self addTopPanel:panel];
   [panel release];
@@ -496,7 +499,8 @@ void setup_mforms_app(WBMainWindow *mwin);
 - (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
 {
   id panel = [_panels objectForKey: [tabViewItem identifier]];
-  
+
+  [self activatePanel: panel];
   if (tabView == topTabView)
   {
     NSMenu *menu = [panel menuBar];
@@ -505,6 +509,8 @@ void setup_mforms_app(WBMainWindow *mwin);
     else
       [NSApp setMainMenu: _defaultMainMenu];
 
+    mforms::App::get()->view_switched();
+
     [tabSwitcher setNeedsDisplay:YES];
   }
   
@@ -1005,19 +1011,14 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (void)firstResponderChanged: (NSResponder*)responder
 {
-  BOOL changedActivePanel = NO;
-
   WBBasePanel *panel= [self panelForResponder:responder];
 
-  if ([panel formBE] != _wbui->get_active_form())
-  {
-    changedActivePanel = YES;
-    [self activatePanel: panel];
-  }
+  BOOL changedActivePanel = panel.formBE != _wbui->get_active_form();
+
   // replace Edit menu with the standard one so that copy/paste works without intervention
   if ([responder isKindOfClass: [NSTextView class]])
   {
-    NSLog(@"restore edit menu");
+    NSLog(@"TODO: restore edit menu");
   }
   else if (!changedActivePanel)
     _wbui->get_command_ui()->revalidate_edit_menu_items();
@@ -1093,7 +1094,7 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (BOOL)closeAllPanels
 {
-  id panelsCopy = [_panels copy];
+  id panelsCopy = [[_panels copy] autorelease];
   id homeTabId = [[topTabView tabViewItemAtIndex: 0] identifier];
   for (id panel in [panelsCopy objectEnumerator])
   {
@@ -1104,7 +1105,6 @@ void setup_mforms_app(WBMainWindow *mwin);
     if (![self closePanel: panel])
       return NO;
   }
-  [panelsCopy release];
   [_panels removeAllObjects];
 
   return YES;
@@ -1216,11 +1216,39 @@ public:
       NSLog(@"set_view_title called for undocked view");
   }
   
-  virtual void get_size(int *w, int *h)
+  virtual std::pair<int, int> get_size()
   {
     NSRect rect = [[self window] frame];
-    *w = NSWidth(rect);
-    *h = NSHeight(rect);
+    return std::make_pair(NSWidth(rect), NSHeight(rect));
+  }
+
+  virtual int view_count()
+  {
+    return [self->topTabView numberOfTabViewItems];
+  }
+
+  virtual mforms::AppView *selected_view()
+  {
+    id view = [[self->topTabView selectedTabViewItem] view];
+    id panel = [self findPanelForView:view inTabView: self->topTabView];
+
+    if ([panel isKindOfClass: [WBMFormsPluginPanel class]])
+    {
+      return [(WBMFormsPluginPanel*)panel appView];
+    }
+    return NULL;
+  }
+
+  virtual mforms::AppView *view_at_index(int index)
+  {
+    id view = [[self->topTabView tabViewItemAtIndex: index] view];
+    id panel = [self findPanelForView:view inTabView: self->topTabView];
+
+    if ([panel isKindOfClass: [WBMFormsPluginPanel class]])
+    {
+      return [(WBMFormsPluginPanel*)panel appView];
+    }
+    return NULL;
   }
 };
 
@@ -1386,7 +1414,7 @@ static base::Color get_system_color(mforms::SystemColor color)
     }
     case mforms::SystemColorContainer:
     {
-      NSColor *c = [[NSColor controlBackgroundColor] colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
+      NSColor *c = [[NSColor colorWithDeviceWhite: 240/255.0 alpha: 1.0] colorUsingColorSpaceName: NSCalibratedRGBColorSpace];
       return base::Color([c redComponent], [c greenComponent], [c blueComponent]);
     }
   }
