@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -51,7 +51,7 @@
   self= [super initWithFrame:NSMakeRect(0,0,1,1)];
   if (self)
   {
-    mOwner= aTable;
+    mOwner = aTable;
     mOwner->set_data(self);
     
     mTableCells= [[NSMutableArray array] retain];
@@ -72,6 +72,11 @@
   [super dealloc];
 }
 
+//--------------------------------------------------------------------------------------------------
+
+STANDARD_MOUSE_HANDLING(self) // Add handling for mouse events.
+
+//--------------------------------------------------------------------------------------------------
 
 - (BOOL)isFlipped
 {
@@ -448,6 +453,11 @@
 
 - (void)addSubview:(NSView*)view left:(int)left right:(int)right top:(int)top bottom:(int)bottom flags:(mforms::TableItemFlags)flags
 {
+  if (left < 0 || left >= mColumnCount || right < left || right > mColumnCount)
+    throw std::invalid_argument("Invalid column coordinate passed to table.add()");
+  if (top < 0 || top >= mRowCount || bottom < top || bottom > mRowCount)
+    throw std::invalid_argument("Invalid row coordinate passed to table.add()");
+  
   [self addSubview:view];
     
   MFTableCell *cell= [[[MFTableCell alloc] init] autorelease];
@@ -584,15 +594,15 @@ static void table_set_column_count(::mforms::Table *self, int count)
 
 static void table_add(::mforms::Table *self, ::mforms::View *child, int left, int right, int top, int bottom, int flags)
 {
-  if ([child->get_data() tag] < 0)
-    NSLog(@"NEGATIVE TAG IN NEW OBJECT tbl %li", (long)[child->get_data() tag]);
+  NSView *view = child->get_data();
+  view.viewFlags |= flags;
   
-  flags|= [child->get_data() tag];
-  
-  // use the tag of the view to store layout flags
-  [child->get_data() setTag:flags];
-  
-  [self->get_data() addSubview:child->get_data() left:left right:right top:top bottom:bottom flags:(mforms::TableItemFlags)flags];
+  [self->get_data() addSubview: view
+                          left: left
+                         right: right
+                           top: top
+                        bottom: bottom
+                         flags: (mforms::TableItemFlags)flags];
 }
 
 
