@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,6 @@
  * 02110-1301  USA
  */
 
-
 #import "MFView.h"
 #import "MFMForms.h"
 #include "base/string_utilities.h"
@@ -67,6 +66,19 @@ static const char *lastDragOperationKey = "lastDragOperationKey";
   objc_setAssociatedObject(self, lastDragOperationKey, @(value), OBJC_ASSOCIATION_RETAIN);
 }
 
+static const char *allowedDragOperationsKey = "allowedDragOperationsKey";
+
+- (mforms::DragOperation)allowedDragOperations
+{
+  NSNumber *value = objc_getAssociatedObject(self, allowedDragOperationsKey);
+  return (mforms::DragOperation)value.intValue;
+}
+
+- (void)setAllowedDragOperations: (mforms::DragOperation)value
+{
+  objc_setAssociatedObject(self, allowedDragOperationsKey, @(value), OBJC_ASSOCIATION_RETAIN);
+}
+
 static const char *acceptableDropFormatsKey = "acceptableDropFormats";
 
 - (NSArray *)acceptableDropFormats
@@ -96,6 +108,19 @@ static const char *dropDelegateKey = "dropDelegate";
   objc_setAssociatedObject(self, dropDelegateKey, @((NSUInteger)delegate), OBJC_ASSOCIATION_RETAIN);
 }
 
+static const char *lastDropPositionKey = "lastDropPositionKey";
+
+- (mforms::DropPosition)lastDropPosition
+{
+  NSNumber *value = objc_getAssociatedObject(self, lastDropPositionKey);
+  return (mforms::DropPosition)value.intValue;
+}
+
+- (void)setLastDropPosition: (mforms::DropPosition)value
+{
+  objc_setAssociatedObject(self, lastDropPositionKey, @(value), OBJC_ASSOCIATION_RETAIN);
+}
+
 //--------------------------------------------------------------------------------------------------
 
 - (bool)handleMouseUp: (NSEvent*)event owner: (mforms::View *)mOwner
@@ -103,18 +128,19 @@ static const char *dropDelegateKey = "dropDelegate";
   NSPoint p = [self convertPoint: [event locationInWindow] fromView: nil];
 
   mforms::MouseButton mouseButton;
-  switch (event.type)
+  switch ([event buttonNumber]) // NSLeftMouseDown etc are NOT buttonNumber constants
   {
-    case NSRightMouseUp:
+    case 1:
       mouseButton = mforms::MouseButtonRight;
       break;
 
-    case NSOtherMouseUp:
-      mouseButton = mforms::MouseButtonOther;
+    case 0:
+      mouseButton = mforms::MouseButtonLeft;
       break;
 
     default:
-      mouseButton = mforms::MouseButtonLeft;
+      mouseButton = mforms::MouseButtonOther;
+      break;
   }
 
   bool handled = false;
@@ -139,18 +165,19 @@ static const char *dropDelegateKey = "dropDelegate";
 {
   NSPoint p = [self convertPoint: [event locationInWindow] fromView: nil];
   mforms::MouseButton mouseButton;
-  switch ([event buttonNumber])
+  switch ([event buttonNumber]) // NSLeftMouseDown etc are NOT buttonNumber constants
   {
-    case NSRightMouseDown:
+    case 1:
       mouseButton = mforms::MouseButtonRight;
       break;
 
-    case NSOtherMouseDown:
-      mouseButton = mforms::MouseButtonOther;
+    case 0:
+      mouseButton = mforms::MouseButtonLeft;
       break;
 
     default:
-      mouseButton = mforms::MouseButtonLeft;
+      mouseButton = mforms::MouseButtonOther;
+      break;
   }
   
   return mOwner->mouse_down(mouseButton, p.x, p.y);
@@ -326,6 +353,18 @@ struct PasteboardDataWrapper {
   }
 };
 
+- (NSDragOperation) draggingSourceOperationMaskForLocal: (BOOL) flag
+{
+  mforms::DragOperation operations = self.allowedDragOperations;
+  NSDragOperation nativeOperations = NSDragOperationNone;
+  if ((operations & mforms::DragOperationMove) == mforms::DragOperationMove)
+    nativeOperations |= NSDragOperationMove;
+  if ((operations & mforms::DragOperationCopy) == mforms::DragOperationCopy)
+    nativeOperations |= NSDragOperationMove;
+
+  return nativeOperations;
+}
+
 - (mforms::DropDelegate *)determineDropDelegate
 {
   mforms::DropDelegate *delegate = self.dropDelegate;
@@ -348,6 +387,14 @@ struct PasteboardDataWrapper {
   // See if we can extract an mforms View from the dragging info which would indicate
   // a drag operation started by mforms.
   id source = sender.draggingSource;
+
+  NSDragOperation nativeOperations = sender.draggingSourceOperationMask;
+  mforms::DragOperation operations = mforms::DragOperationNone;
+  if ((nativeOperations & NSDragOperationMove) == NSDragOperationMove)
+    operations |= mforms::DragOperationMove;
+  if ((nativeOperations & NSDragOperationCopy) == NSDragOperationCopy)
+    operations |= mforms::DragOperationCopy;
+
   mforms::View *view = NULL;
   if ([source respondsToSelector: @selector(mformsObject)])
     view = dynamic_cast<mforms::View*>([source mformsObject]);
@@ -367,7 +414,8 @@ struct PasteboardDataWrapper {
   }
 
   NSDragOperation result = NSDragOperationNone;
-  mforms::DragOperation operation = delegate->drag_over(view, base::Point(location.x, location.y), formats);
+  mforms::DragOperation operation = delegate->drag_over(view, base::Point(location.x, location.y),
+                                                        operations, formats);
   self.lastDragOperation = operation;
   if ((operation & mforms::DragOperationCopy) != 0)
     result |= NSDragOperationCopy;
@@ -386,6 +434,14 @@ struct PasteboardDataWrapper {
   // See if we can extract an mforms View from the dragging info which would indicate
   // a drag operation started by mforms.
   id source = sender.draggingSource;
+
+  NSDragOperation nativeOperations = sender.draggingSourceOperationMask;
+  mforms::DragOperation operations = mforms::DragOperationNone;
+  if ((nativeOperations & NSDragOperationMove) == NSDragOperationMove)
+    operations |= mforms::DragOperationMove;
+  if ((nativeOperations & NSDragOperationCopy) == NSDragOperationCopy)
+    operations |= mforms::DragOperationCopy;
+
   mforms::View *view = NULL;
   if ([source respondsToSelector: @selector(mformsObject)])
     view = dynamic_cast<mforms::View*>([source mformsObject]);
@@ -397,7 +453,8 @@ struct PasteboardDataWrapper {
     if ([entry isEqualToString: NSStringPboardType])
     {
       NSString *text = [pasteboard stringForType: NSStringPboardType];
-      if (delegate->text_dropped(view, base::Point(location.x, location.y), [text UTF8String]) != mforms::DragOperationNone)
+      if (delegate->text_dropped(view, base::Point(location.x, location.y), operations,
+                                 [text UTF8String]) != mforms::DragOperationNone)
         return YES;
     }
     else
@@ -407,7 +464,8 @@ struct PasteboardDataWrapper {
         std::vector<std::string> names;
         for (NSString *name in fileNames)
           names.push_back([name UTF8String]);
-        if (names.size() > 0 && delegate->files_dropped(view, base::Point(location.x, location.y), names) != mforms::DragOperationNone)
+        if (names.size() > 0 && delegate->files_dropped(view, base::Point(location.x, location.y),
+                                                        operations, names) != mforms::DragOperationNone)
           return YES;
       }
       else
@@ -416,7 +474,8 @@ struct PasteboardDataWrapper {
         void *data = [pasteboard nativeDataForTypeAsString: entry];
         if (data != NULL)
         {
-          if (delegate->data_dropped(view, base::Point(location.x, location.y), data, [entry UTF8String]) != mforms::DragOperationNone)
+          if (delegate->data_dropped(view, base::Point(location.x, location.y), operations, data,
+                                     [entry UTF8String]) != mforms::DragOperationNone)
             return YES;
         }
       }
@@ -425,9 +484,17 @@ struct PasteboardDataWrapper {
   return NO;
 }
 
+- (void)draggingEnded: (id <NSDraggingInfo>)sender
+{
+  self.lastDropPosition = mforms::DropPositionUnknown;
+}
+
 - (mforms::DragOperation)startDragWithText: (NSString *)text
                                    details: (mforms::DragDetails)details
 {
+  self.allowedDragOperations = details.allowedOperations;
+  self.lastDropPosition = mforms::DropPositionUnknown;
+
   NSPasteboard *pasteboard = NSPasteboard.generalPasteboard;
   [pasteboard clearContents];
   [pasteboard setString: text forType: NSStringPboardType];
@@ -503,6 +570,9 @@ struct PasteboardDataWrapper {
                                    details: (mforms::DragDetails)details
                                     format: (NSString *)format
 {
+  self.allowedDragOperations = details.allowedOperations;
+  self.lastDropPosition = mforms::DropPositionUnknown;
+
   NSPasteboard *pasteboard = NSPasteboard.generalPasteboard;
   [pasteboard clearContents];
   [pasteboard writeNativeData: data typeAsString: format];
@@ -651,7 +721,7 @@ static int view_get_y(::mforms::View *self)
 static void view_set_size(::mforms::View *self, int w, int h)
 {
   id view = self->get_data();
-  if (view)
+  if (view != nil && [view respondsToSelector: @selector(setFixedFrameSize:)])
   {
     [view setFixedFrameSize: NSMakeSize(w,h)];
   }
@@ -782,6 +852,19 @@ static bool view_is_shown(::mforms::View *self)
   return false;
 }
 
+static bool view_is_fully_visible(::mforms::View *self)
+{
+  NSView *view = self->get_data();
+  if (view)
+  {
+    if (view.window == nil)
+      return false;
+
+    return !view.isHiddenOrHasHiddenAncestor;
+  }
+  return false;
+}
+
 static void view_set_tooltip(::mforms::View *self, const std::string &text)
 {
   id view = self->get_data();
@@ -907,6 +990,17 @@ static void view_focus(::mforms::View *self)
   [[view window] makeFirstResponder: view];
 }
 
+static bool view_has_focus(::mforms::View *self)
+{
+  id view = self->get_data();
+  id firstResponder = [[view window] firstResponder];
+  if (firstResponder == view)
+    return true;
+  if ([firstResponder respondsToSelector: @selector(delegate)] && [firstResponder delegate] == view)
+    return true;
+  return false;
+}
+
 static void register_drop_formats(mforms::View *self, mforms::DropDelegate *target, const std::vector<std::string> &formats)
 {
   NSMutableArray *list = [[[NSMutableArray alloc] init] autorelease];
@@ -921,6 +1015,7 @@ static void register_drop_formats(mforms::View *self, mforms::DropDelegate *targ
         [list addObject: [NSString stringWithUTF8String: formats[i].c_str()]];
   }
   NSView *view = self->get_data();
+
   view.acceptableDropFormats = list;
   view.dropDelegate = target;
 }
@@ -937,10 +1032,16 @@ static mforms::DragOperation view_drag_data(mforms::View *self, mforms::DragDeta
 {
   NSView *view = self->get_data();
   return [view startDragWithData: data
-                        details: details
+                         details: details
                           format: [NSString stringWithUTF8String: format.c_str()]];
 }
 
+static mforms::DropPosition view_get_drop_position(mforms::View *self)
+{
+  NSView *view = self->get_data();
+  return view.lastDropPosition;
+}
+
 void cf_view_init()
 {
   ::mforms::ControlFactory *f= ::mforms::ControlFactory::get_instance();
@@ -962,6 +1063,7 @@ void cf_view_init()
 
   f->_view_impl.show                 = &view_show;
   f->_view_impl.is_shown             = &view_is_shown;
+  f->_view_impl.is_fully_visible     = &view_is_fully_visible;
   
   f->_view_impl.set_tooltip          = &view_set_tooltip;
   f->_view_impl.set_name             = &view_set_name;
@@ -981,8 +1083,10 @@ void cf_view_init()
 
   f->_view_impl.flush_events         = &view_flush_events;
   f->_view_impl.focus                = &view_focus;
+  f->_view_impl.has_focus            = &view_has_focus;
 
   f->_view_impl.register_drop_formats = &register_drop_formats;
   f->_view_impl.drag_text            = &view_drag_text;
   f->_view_impl.drag_data            = &view_drag_data;
+  f->_view_impl.get_drop_position    = &view_get_drop_position;
 }
