@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,14 +20,21 @@
 //#include "config.h"
 #if defined(HAVE_GNOME_KEYRING) || defined(HAVE_OLD_GNOME_KEYRING)
 extern "C" {
+// gnome-keyring has been deprecated in favor of libsecret
+// More informations can be found here  https://mail.gnome.org/archives/commits-list/2013-October/msg08876.html
+// Below defines will turn off deprecations and allow build with never Gnome until we will not move to libsecret.
+  #define GNOME_KEYRING_DEPRECATED
+  #define GNOME_KEYRING_DEPRECATED_FOR(x)
   #include <gnome-keyring.h>
 };
   #include <string.h>
   #define USE_KEYRING 1
 #endif
 
-#include <sys/mman.h>
 #include <glib/gstdio.h>
+#ifdef HAVE_PRCTL_H
+#include <sys/prctl.h>
+#endif
 
 // this is broken
 //#undef USE_KEYRING
@@ -36,6 +43,7 @@ extern "C" {
 #include "base/file_functions.h"
 #include "base/file_utilities.h"
 #include "base/log.h"
+#include "base/string_utilities.h"
 #include "../lf_utilities.h"
 #include "gtk_helpers.h"
 #include "mforms.h"
@@ -43,154 +51,40 @@ extern "C" {
 namespace mforms {
 namespace gtk {
 
-class PasswordCache
+/**
+  * Get the current active window for this application
+  */
+GtkWindow *get_current_window()
 {
-  char *storage;
-  size_t storage_len;
-  size_t storage_size;
+  GList *window_list = gtk_window_list_toplevels();
+  do
+  {
+    GtkWindow *wnd = (GtkWindow *)window_list->data;
+    
+    if (gtk_window_is_active(wnd))
+      return wnd;
+  } while((window_list = g_list_next(window_list)) != NULL);
 
-  PasswordCache()
-  {
-    storage_len = 0;
-    storage_size = 4*1024;
-    storage = (char*)malloc(storage_size);
-    if (!storage)
-      perror("malloc password cache");
-    else
-      if (mlock(storage, storage_size) < 0)
-      {
-        perror("mlock password cache");
-        free(storage);
-        storage = NULL;
-      }
-  }
-  static PasswordCache instance;
-public:
-  static PasswordCache *get()
-  {
-    return &instance;
-  }
+  return NULL;
+}
+
+/**
+ *  Try to set this dialog as transcient to the parent window. Setting this
+ *  to the main window could stick this dialog to the wrong window (leaving
+ *  a window in between them in the cases like popping a dialog in the
+ *  scripting shell.
+ *  To find the proper parent window, we're searching for the current active window
+ *  on this application.
+ */
+void set_dialog_transcient(Gtk::MessageDialog &dialog)
+{
+  GtkWindow *parent_window = get_current_window();
   
-  ~PasswordCache()
-  {
-    if (storage)
-    {
-      memset(storage, 0, storage_size);
-      if (munlock(storage, storage_size) < 0)
-        perror("munlock password cache");
-      free(storage);
-    }
-  }
-
-  void add_password(const std::string &service, const std::string &account, const char *password)
-  {
-    if (storage)
-    {
-      if (!password) password = "";
-      
-      {
-        const char *opassword = find_password(service, account);
-        if (opassword)
-        {
-          if (strcmp(password, opassword) == 0)
-            return;
-          remove_password(service, account);
-        }
-      }
-
-      size_t reclen = sizeof(size_t) + service.size() + 1 + account.size() + 1 + strlen(password) + 1;
-      
-      // increase buffer size if new record doesnt fit
-      while (storage_len + reclen > storage_size)
-      {
-        char *new_block;
-        size_t new_size = storage_size + 4*1024;
-
-        new_block = (char*)malloc(new_size);
-        if (new_block)
-        {
-          if (mlock(new_block, new_size) < 0)
-          {
-            perror("mlock password cache");
-            free(new_block);
-            throw std::runtime_error("Could not increase password cache size");
-          }
-          memcpy(new_block, storage, storage_len);
-          memset(storage, 0, storage_size);
-          if (munlock(storage, storage_size) < 0)
-            perror("munlock password cache");
-          free(storage);
-          storage = new_block;
-          storage_size = new_size;
-        }
-        else
-          throw std::runtime_error("Could not increase password cache size");
-      }
-
-      // store length of this record
-      *(size_t*)(storage + storage_len) = reclen;
-      storage_len += sizeof(reclen);
-      // store contents
-      memcpy(storage + storage_len, service.data(), service.size()+1);
-      storage_len += service.size()+1;
-      memcpy(storage + storage_len, account.data(), account.size()+1);
-      storage_len += account.size()+1;
-      memcpy(storage + storage_len, password, strlen(password)+1);
-      storage_len += strlen(password)+1;
-    }
-    else
-      throw std::runtime_error("Password storage is not available");
-  }
-  
-  size_t find_block(const std::string &service, const std::string &account)
-  {
-    size_t offset = 0;
-    while (offset < storage_len)
-    {
-      size_t recsize = *(size_t*)(storage+offset);
-      const char *recservice = storage+offset+sizeof(recsize);
-      const char *recaccount = storage+offset+sizeof(recsize)+strlen(recservice)+1;
-
-      if (strcmp(recservice, service.c_str()) == 0 &&
-          strcmp(recaccount, account.c_str()) == 0)
-        return offset;
-
-      offset += recsize;
-    }
-    return (size_t)-1;
-  }
-
-  void remove_password(const std::string &service, const std::string &account)
-  {
-    if (storage)
-    {
-      size_t offset = find_block(service, account);
-      if (offset != (size_t)-1)
-      {
-        size_t recsize = *(size_t*)(storage+offset);
-
-        memmove(storage+offset, storage+offset+recsize, storage_len-recsize);
-        storage_len -= recsize;
-      }
-    }
-  }
-
-  const char *find_password(const std::string &service, const std::string &account)
-  {
-    if (storage)
-    {
-      size_t offset = find_block(service, account);
-      if (offset != (size_t)-1)
-      {
-        const char *pwd = storage + offset + sizeof(size_t) + service.size()+1 + account.size()+1;
-        return pwd;
-      }
-    }
-    return 0;
-  }
-};
-PasswordCache PasswordCache::instance;
-
+  //  Check if a parent was found and only set transcient if it was. Passing
+  //  a NULL parent would remove the transcient flag.
+  if (get_current_window() != NULL)
+    gtk_window_set_transient_for(((Gtk::Window *)&dialog)->gobj(), parent_window);
+}
 //--------------------------------------------------------------------------------
   
 
@@ -207,10 +101,8 @@ int UtilitiesImpl::show_message(const std::string &title, const std::string &tex
   if (!other.empty())
     dlg.add_button(other, mforms::ResultOther);
 
-  //unnecessary.. apparently newer window managers set this automatically for dialogs
-  //and setting this to main_window explicitly will break that and cause the dialog
-  //to be assigned to the wrong window when a non-main window is the actual owner
-  //dlg.set_transient_for(*get_mainwindow());
+  set_dialog_transcient(dlg);
+
   int r= dlg.run();
   if (r == Gtk::RESPONSE_DELETE_EVENT)
     return mforms::ResultCancel;
@@ -231,7 +123,8 @@ int UtilitiesImpl::show_error(const std::string &title, const std::string &text,
   if (!other.empty())
     dlg.add_button(other, mforms::ResultOther);
 
-//  dlg.set_transient_for(*get_mainwindow());
+  set_dialog_transcient(dlg);
+
   int r= dlg.run();
   if (r == Gtk::RESPONSE_DELETE_EVENT)
     return mforms::ResultCancel;
@@ -251,7 +144,8 @@ int UtilitiesImpl::show_warning(const std::string &title, const std::string &tex
   if (!other.empty())
     dlg.add_button(other, mforms::ResultOther);
 
-  //dlg.set_transient_for(*get_mainwindow());
+  set_dialog_transcient(dlg);
+  
   int r= dlg.run();
   if (r == Gtk::RESPONSE_DELETE_EVENT)
     return mforms::ResultCancel;
@@ -285,7 +179,9 @@ int UtilitiesImpl::show_message_with_checkbox(const std::string &title, const st
   if (!other.empty())
     dlg.add_action_widget(*Gtk::manage(new Gtk::Button(other)), mforms::ResultOther);
   dlg.show_all();
- // dlg.set_transient_for(*get_mainwindow());
+
+  set_dialog_transcient(dlg);
+
   int r= dlg.run();
   if (r == Gtk::RESPONSE_DELETE_EVENT)
     return mforms::ResultCancel;
@@ -308,7 +204,7 @@ void UtilitiesImpl::open_url(const std::string &url)
                                          G_URI_RESERVED_CHARS_GENERIC_DELIMITERS 
                                          G_URI_RESERVED_CHARS_SUBCOMPONENT_DELIMITERS, 
                                          FALSE);
-  
+
   const gchar *argv[] = {
     "xdg-open",
     quoted_url,
@@ -373,24 +269,54 @@ std::string UtilitiesImpl::get_special_folder(mforms::FolderType type)
 }
 
 //------------------------------------------------------------------------------
-inline bool run_slot(const boost::function<bool ()> slot)
+static std::map<TimeoutHandle, sigc::connection> timeouts;
+static TimeoutHandle last_timeout_handle = 0;
+static base::Mutex timeout_mutex;
+
+inline bool run_slot(const boost::function<bool ()> slot, TimeoutHandle handle)
 {
-  return slot();
+  if (!slot())
+  {
+    base::MutexLock lock(timeout_mutex);
+    std::map<TimeoutHandle, sigc::connection>::iterator it;
+    if ((it = timeouts.find(handle)) != timeouts.end())
+      timeouts.erase(it);
+    return false;
+  }
+  return true;
 }
 
 //------------------------------------------------------------------------------
-void UtilitiesImpl::add_timeout(float interval, const boost::function<bool ()> &slot)
+TimeoutHandle UtilitiesImpl::add_timeout(float interval, const boost::function<bool ()> &slot)
 {
   try
   {
     if (slot)
-      Glib::signal_timeout().connect(sigc::bind(sigc::ptr_fun(run_slot), slot), interval*1000);
+    {
+      base::MutexLock lock(timeout_mutex);
+      ++last_timeout_handle;
+      timeouts[last_timeout_handle] = Glib::signal_timeout().connect(sigc::bind(sigc::ptr_fun(run_slot), slot, last_timeout_handle), interval*1000);
+      return last_timeout_handle;
+    }
   }
   catch (std::exception &exc)
   {
     static const char* const default_log_domain = "Utilities";
     log_exception("Utilities: exception in add timeout function", exc);
   }
+  return 0;
+}
+
+
+void UtilitiesImpl::cancel_timeout(TimeoutHandle h)
+{
+  base::MutexLock lock(timeout_mutex);
+  std::map<TimeoutHandle, sigc::connection>::iterator it;
+  if ((it = timeouts.find(h)) != timeouts.end())
+  {
+    it->second.disconnect();
+    timeouts.erase(it);
+  }
 }
 
 
@@ -402,7 +328,6 @@ void UtilitiesImpl::store_password(const std::string &service, const std::string
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    PasswordCache::get()->add_password(service, account, password.c_str());
     return;
   }
 
@@ -434,11 +359,7 @@ bool UtilitiesImpl::find_password(const std::string &service, const std::string
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    const char *pwd = PasswordCache::get()->find_password(service, account);
-    if (!pwd)
-      return false;
-    password = pwd;
-    return true;
+    return false;
   }
 
   bool ret = false;
@@ -461,6 +382,14 @@ bool UtilitiesImpl::find_password(const std::string &service, const std::string
                                                             "account", account.c_str(),
                                                             NULL);
 
+  if ( res == GNOME_KEYRING_RESULT_CANCELLED)
+  {
+    if (kpwd)
+      gnome_keyring_free_password(kpwd);
+    kpwd = 0;
+    throw grt::user_cancelled("User cancelled password lookup.");
+  }
+
   if ( res != GNOME_KEYRING_RESULT_OK && res != GNOME_KEYRING_RESULT_NO_MATCH )
   {
     if (kpwd)
@@ -485,7 +414,6 @@ void UtilitiesImpl::forget_password(const std::string &service, const std::strin
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    PasswordCache::get()->remove_password(service, account);
     return;
   }
 
@@ -499,13 +427,14 @@ void UtilitiesImpl::forget_password(const std::string &service, const std::strin
   wb_pwd_schema.attributes[1].type = GNOME_KEYRING_ATTRIBUTE_TYPE_STRING;
   // attributes list must be terminated by a pair {0,0}, since we zeroed the whole struct we're safe here
 
-  gchar *kpwd = 0;
-
   GnomeKeyringResult res = gnome_keyring_delete_password_sync(&wb_pwd_schema,
                                                             "service", service.c_str(),
                                                             "account", account.c_str(),
                                                             NULL);
 
+  if ( res == GNOME_KEYRING_RESULT_CANCELLED)
+    throw grt::user_cancelled("User cancelled password lookup.");
+
   if ( res != GNOME_KEYRING_RESULT_OK && res != GNOME_KEYRING_RESULT_NO_MATCH)
     throw std::runtime_error(std::string("forget_password ") + gnome_keyring_result_to_message(res));
 }
@@ -542,7 +471,6 @@ void UtilitiesImpl::store_password(const std::string &service, const std::string
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    PasswordCache::get()->add_password(service, account, password.c_str());
     return;
   }
 
@@ -582,6 +510,13 @@ static guint32 find_password_and_id(const std::string &service, const std::strin
                                                                   attrs,
                                                                   &found_items);
 
+  if (result == GNOME_KEYRING_RESULT_CANCELLED)
+  {
+    if (found_items)
+      gnome_keyring_found_list_free(found_items);
+    throw grt::user_cancelled("User cancelled password lookup.");
+  }
+
   if (result != GNOME_KEYRING_RESULT_OK)
   {
     if (found_items)
@@ -607,11 +542,7 @@ bool UtilitiesImpl::find_password(const std::string &service, const std::string
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    const char *pwd = PasswordCache::get()->find_password(service, account);
-    if (!pwd)
-      return false;
-    password = pwd;
-    return true;
+    return false;
   }
 
   return find_password_and_id(service, account, password);
@@ -622,7 +553,6 @@ void UtilitiesImpl::forget_password(const std::string &service, const std::strin
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    PasswordCache::get()->remove_password(service, account);
     return;
   }
 
@@ -640,22 +570,16 @@ void UtilitiesImpl::forget_password(const std::string &service, const std::strin
 
 void UtilitiesImpl::store_password(const std::string &service, const std::string &account, const std::string &password)
 {
-  PasswordCache::get()->add_password(service, account, password.c_str());
 }
 
 bool UtilitiesImpl::find_password(const std::string &service, const std::string &account, std::string &password)
 {
   g_message("no gnome keyring support");
-  const char *pwd = PasswordCache::get()->find_password(service, account);
-  if (!pwd)
-    return false;
-  password = pwd;
-  return true;
+  return false;
 }
 
 void UtilitiesImpl::forget_password(const std::string &service, const std::string &account)
 {
-  PasswordCache::get()->remove_password(service, account);
 }
 
 #endif // !USE_KEYRING
@@ -678,6 +602,7 @@ class TransparentMessage : public Gtk::Window
   public:
     TransparentMessage();
     void show_message(const std::string& title, const std::string& text, const sigc::slot<bool> &cancel_slot=sigc::slot<bool>());
+    void run();
     void stop();
     bool response;
     bool running_modal;
@@ -685,6 +610,7 @@ class TransparentMessage : public Gtk::Window
     Gtk::Button *cancel_button;
     sigc::slot<bool> cancel_slot;
     Glib::Mutex mutex;
+    bool _is_runing;
 
     virtual bool on_button_release_event(GdkEventButton* ev);
     void cancel_clicked();
@@ -692,7 +618,7 @@ class TransparentMessage : public Gtk::Window
 
 //------------------------------------------------------------------------------
 TransparentMessage::TransparentMessage()
-  : response(false)
+  : response(false), _is_runing(false)
 {
   Gtk::Window *owner = get_mainwindow();
   if (owner)
@@ -783,13 +709,15 @@ void TransparentMessage::show_message(const std::string& title, const std::strin
 #if GTK_VERSION_GE(2,12)
   window->set_opacity(0.85);
 #endif
+  show_all();
   window->process_updates(true);
-
-  show_now();
-
-  window->freeze_updates(); // Added to stop idle window update requests storm in gnome-3
 }
 
+void TransparentMessage::run()
+{
+  _is_runing = true;
+  Gtk::Main::run();
+}
 
 //------------------------------------------------------------------------------
 void TransparentMessage::stop()
@@ -799,7 +727,11 @@ void TransparentMessage::stop()
   {
     unrealize();
     running_modal = false;
-    Gtk::Main::quit();
+    if (_is_runing)
+    {
+      Gtk::Main::quit();
+      _is_runing = false;
+    }
   }
   response= true;
 }
@@ -814,8 +746,12 @@ void TransparentMessage::cancel_clicked()
     {
       unrealize();
       running_modal = false;
-      Gtk::Main::quit();
-      hide();
+      if (_is_runing)
+      {
+        Gtk::Main::quit();
+        hide();
+        _is_runing = false;
+      }
     }
     response = false;
   }
@@ -875,9 +811,10 @@ bool UtilitiesImpl::move_to_trash(const std::string &path)
 //------------------------------------------------------------------------------
 void UtilitiesImpl::reveal_file(const std::string &path)
 {
+  std::string dirname = base::dirname(path);
   const gchar *argv[] = {
     "xdg-open",
-    base::dirname(path).c_str(),
+    dirname.c_str(),
     NULL 
   };
   GError *error = NULL;
@@ -893,12 +830,28 @@ void UtilitiesImpl::reveal_file(const std::string &path)
 }
 
 
+void UtilitiesImpl::set_thread_name(const std::string &name)
+{
+#ifdef HAVE_PRCTL_H 
+  if (!name.empty())
+    prctl(PR_SET_NAME, name.c_str(),0,0,0);
+#endif
+}
+
+void UtilitiesImpl::beep()
+{
+  get_mainwindow()->get_window()->beep();
+}
+
 //------------------------------------------------------------------------------
 
 
 bool UtilitiesImpl::run_cancelable_wait_message(const std::string &title, const std::string &text,
                                                  const boost::function<void ()> &start_task, const boost::function<bool ()> &cancel_task)
 {
+  if (!start_task)
+    throw std::invalid_argument("start_task param cannot be empty");
+
   if (!tmc)
     tmc = new TransparentMessage();
   if (tmc)
@@ -907,7 +860,8 @@ bool UtilitiesImpl::run_cancelable_wait_message(const std::string &title, const
     tmc->running_modal = true;
 
     Glib::signal_idle().connect(sigc::bind_return(start_task, false));
-    Gtk::Main::run();
+
+    tmc->run();
     bool response = tmc->response;
     delete tmc;
     tmc = 0;
@@ -929,18 +883,19 @@ void UtilitiesImpl::stop_cancelable_wait_message()
   }
 }
 
+static std::map<std::string, Glib::RefPtr<Gdk::Pixbuf> > icon_cache;
    
 Glib::RefPtr<Gdk::Pixbuf> UtilitiesImpl::get_cached_icon(const std::string &icon)
 {
-  std::map<std::string, Glib::RefPtr<Gdk::Pixbuf> > cache;
 
-  if (cache.find(icon) != cache.end())
-    return cache[icon];
+
+  if (icon_cache.find(icon) != icon_cache.end())
+    return icon_cache[icon];
 
   if (icon == "folder")
   {
     Glib::RefPtr<Gdk::Pixbuf> pix = get_mainwindow()->render_icon(Gtk::Stock::DIRECTORY, Gtk::ICON_SIZE_MENU);
-    cache[icon] = pix;
+    icon_cache[icon] = pix;
     return pix;
   }
   else
@@ -948,8 +903,8 @@ Glib::RefPtr<Gdk::Pixbuf> UtilitiesImpl::get_cached_icon(const std::string &icon
     std::string path = mforms::App::get()->get_resource_path(icon);
     if (!path.empty() && g_file_test(path.c_str(), G_FILE_TEST_IS_REGULAR))
     {
-      cache[icon] = Gdk::Pixbuf::create_from_file(path);
-      return cache[icon];
+      icon_cache[icon] = Gdk::Pixbuf::create_from_file(path);
+      return icon_cache[icon];
     }
     else
       g_warning("Can't find icon %s", icon.c_str());
@@ -958,6 +913,79 @@ Glib::RefPtr<Gdk::Pixbuf> UtilitiesImpl::get_cached_icon(const std::string &icon
 }
 
 
+//------------------------------------------------------------------------------
+#include <pango/pangoft2.h>
+
+class FontMeasurement
+{
+public:
+  PangoLayout *_layout;
+  FontMeasurement(PangoLayout *pango) : _layout(pango)
+  {
+  }
+
+//  FontMeasurement(const FontMeasurement &other)
+//  {
+//    if (other._layout!=NULL)
+//      g_object_ref(other._layout);
+//  }
+//
+//  void operator=(const FontMeasurement &other)
+//  {
+//    if (other._layout!=NULL)
+//      g_object_ref(other._layout);
+//  }
+
+  ~FontMeasurement()
+  {
+    if (_layout != NULL)
+      g_object_unref (_layout);
+  }
+};
+
+std::map<std::string, FontMeasurement*> FontMeasurementDescriptors;
+
+double UtilitiesImpl::get_text_width(const std::string &text, const std::string &font_desc)
+{
+  std::string font;
+  float size = 0;
+  bool bold = false;
+  bool italic = false;
+  
+  if (!base::parse_font_description(font_desc, font, size, bold, italic))
+    return 0;
+
+  PangoLayout *layout = NULL;
+  std::map<std::string, FontMeasurement*>::iterator it = FontMeasurementDescriptors.find(font_desc);
+  if (it == FontMeasurementDescriptors.end())
+  {
+    PangoFontDescription *font_description = pango_font_description_new ();
+    FontMeasurement *font_measurement = new FontMeasurement(pango_layout_new(get_mainwindow()->get_pango_context()->gobj()));
+
+    pango_font_description_set_family (font_description, font.c_str());
+    pango_font_description_set_style (font_description, italic ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL);
+    pango_font_description_set_variant (font_description, PANGO_VARIANT_NORMAL);
+    pango_font_description_set_weight (font_description, bold ?  PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL);
+    pango_font_description_set_stretch (font_description, PANGO_STRETCH_NORMAL);
+    pango_font_description_set_size (font_description, size * PANGO_SCALE);
+    pango_layout_set_font_description (font_measurement->_layout, font_description);
+    pango_font_description_free(font_description);
+
+    FontMeasurementDescriptors[font_desc] = font_measurement;
+    layout = font_measurement->_layout;
+  }
+  else
+    layout = it->second->_layout;
+
+  pango_layout_set_text (layout, text.c_str(), -1);
+
+  int width = 0;
+  pango_layout_get_pixel_size (layout, &width, NULL);
+  
+  return (double)width;
+  
+}
+
 //------------------------------------------------------------------------------
 
 void UtilitiesImpl::init()
@@ -977,6 +1005,7 @@ void UtilitiesImpl::init()
   f->_utilities_impl.get_clipboard_text= &UtilitiesImpl::get_clipboard_text;
   f->_utilities_impl.open_url= &UtilitiesImpl::open_url;
   f->_utilities_impl.add_timeout= &UtilitiesImpl::add_timeout;
+  f->_utilities_impl.cancel_timeout= &UtilitiesImpl::cancel_timeout;
   f->_utilities_impl.get_special_folder= &UtilitiesImpl::get_special_folder;
   f->_utilities_impl.store_password= &UtilitiesImpl::store_password;
   f->_utilities_impl.find_password= &UtilitiesImpl::find_password;
@@ -984,7 +1013,10 @@ void UtilitiesImpl::init()
   f->_utilities_impl.move_to_trash= &UtilitiesImpl::move_to_trash;
   f->_utilities_impl.reveal_file= &UtilitiesImpl::reveal_file;
   f->_utilities_impl.perform_from_main_thread = &MainThreadRequestQueue::perform;
+  f->_utilities_impl.set_thread_name = &UtilitiesImpl::set_thread_name;
+  f->_utilities_impl.beep = &UtilitiesImpl::beep;
 
+  f->_utilities_impl.get_text_width = &UtilitiesImpl::get_text_width;
   MainThreadRequestQueue::get(); // init from main thread
 }
 
