@@ -17,7 +17,7 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
+#include "base/util_functions.h"
 
 #include "grtui/grt_wizard_plugin.h"
 #include "grtui/wizard_progress_page.h"
@@ -30,6 +30,7 @@
 #include "db_mysql_sql_export.h"
 #include "base/string_utilities.h"
 #include "mforms/fs_object_selector.h"
+#include "grts/structs.workbench.h"
 
 using namespace grtui;
 using namespace mforms;
@@ -77,16 +78,19 @@ public:
     _options_box.add(&_generate_drop_check, false, false);
     _generate_drop_schema_check.set_text(_("Generate DROP SCHEMA"));
     _options_box.add(&_generate_drop_schema_check, false, false);
+
     _skip_foreign_keys_check.set_text(_("Skip Creation of FOREIGN KEYS"));
     _options_box.add(&_skip_foreign_keys_check, false, false);
     scoped_connect(_skip_foreign_keys_check.signal_clicked(),boost::bind(&ExportInputPage::SkipFKToggled, this));
     _skip_FK_indexes_check.set_text(_("Skip creation of FK Indexes as well"));
     _options_box.add(&_skip_FK_indexes_check, false, false);
+
     _omit_schema_qualifier_check.set_text(_("Omit Schema Qualifier in Object Names"));
     _options_box.add(&_omit_schema_qualifier_check, false, false);
     scoped_connect(_omit_schema_qualifier_check.signal_clicked(),boost::bind(&ExportInputPage::OmitSchemaToggled, this));
     _generate_use_check.set_text(_("Generate USE statements"));
     _options_box.add(&_generate_use_check, false, false);
+
     _generate_create_index_check.set_text(_("Generate Separate CREATE INDEX Statements"));
     _options_box.add(&_generate_create_index_check, false, false);
     _generate_show_warnings_check.set_text(_("Add SHOW WARNINGS After Every DDL Statement"));
@@ -95,6 +99,7 @@ public:
     _options_box.add(&_skip_users_check, false, false);
     _no_view_placeholders.set_text(_("Don't create view placeholder tables."));
     _options_box.add(&_no_view_placeholders, false, false);
+    
     _generate_insert_check.set_text(_("Generate INSERT Statements for Tables"));
     _options_box.add(&_generate_insert_check, false, false);
     _no_FK_for_inserts.set_text(_("Disable FK checks for inserts"));
@@ -105,17 +110,17 @@ public:
 
     add(&_options, false, false);
     
-    _generate_drop_check.set_active(form->module()->document_int_data("generate_drop", false));
-    _generate_drop_schema_check.set_active(form->module()->document_int_data("generate_schema_drop", 0));
-    _skip_foreign_keys_check.set_active(form->module()->document_int_data("skip_foreign_keys", false));
-    _skip_FK_indexes_check.set_active(form->module()->document_int_data("SkipFKIndexes", false));
-    _omit_schema_qualifier_check.set_active(form->module()->document_int_data("omit_schema_qualifier", false));
-    _generate_create_index_check.set_active(form->module()->document_int_data("generate_create_index", false));
-    _generate_show_warnings_check.set_active(form->module()->document_int_data("generate_show_warnings", false));
-    _skip_users_check.set_active(form->module()->document_int_data("skip_users", false));
-    _no_view_placeholders.set_active(form->module()->document_int_data("no_vew_placeholders", false));
-    _generate_insert_check.set_active(form->module()->document_int_data("generate_insert", false));
-    _generate_use_check.set_active(form->module()->document_int_data("generate_use", false));
+    _generate_drop_check.set_active(form->module()->document_int_data("generate_drop", false) != 0);
+    _generate_drop_schema_check.set_active(form->module()->document_int_data("generate_schema_drop", 0) != 0);
+    _skip_foreign_keys_check.set_active(form->module()->document_int_data("skip_foreign_keys", false) != 0);
+    _skip_FK_indexes_check.set_active(form->module()->document_int_data("SkipFKIndexes", false) != 0);
+    _omit_schema_qualifier_check.set_active(form->module()->document_int_data("omit_schema_qualifier", false) != 0);
+    _generate_create_index_check.set_active(form->module()->document_int_data("generate_create_index", false) != 0);
+    _generate_show_warnings_check.set_active(form->module()->document_int_data("generate_show_warnings", false) != 0);
+    _skip_users_check.set_active(form->module()->document_int_data("skip_users", false) != 0);
+    _no_view_placeholders.set_active(form->module()->document_int_data("no_vew_placeholders", false) != 0);
+    _generate_insert_check.set_active(form->module()->document_int_data("generate_insert", false) != 0);
+    _generate_use_check.set_active(form->module()->document_int_data("generate_use", false) != 0);
     _generate_use_check.set_enabled(_omit_schema_qualifier_check.get_active());
     _skip_FK_indexes_check.set_enabled(_skip_foreign_keys_check.get_active());
   }
@@ -295,26 +300,45 @@ protected:
 
   virtual bool advance()
   {
+    std::string header;
+
+    db_CatalogRef catalog(_export_be->get_catalog());
+    if (catalog.is_valid() && catalog->owner().is_valid() && catalog->owner()->owner().is_valid())
+    {
+      workbench_DocumentRef doc(workbench_DocumentRef::cast_from(catalog->owner()->owner()));
+      std::string description = doc->info()->description();
+
+      base::replace(description, "\n", "\n-- ");
+
+      header = "-- MySQL Script generated by MySQL Workbench\n";
+      header += "-- " + base::fmttime(0, "%c") + "\n";
+      header += "-- Model: "+*doc->info()->caption()+"    Version: "+*doc->info()->version()+"\n";
+      if (!description.empty())
+        header += "\n-- " + description + "\n\n";
+    }
+
     _export_be->set_option("OutputFileName", values().get_string("OutputFileName"));
-    _export_be->set_option("GenerateDrops", values().get_int("GenerateDrops"));
-    _export_be->set_option("GenerateSchemaDrops", values().get_int("GenerateSchemaDrops"));
-    _export_be->set_option("SkipForeignKeys", values().get_int("SkipForeignKeys"));
-    _export_be->set_option("SkipFKIndexes", values().get_int("SkipFKIndexes"));
-    _export_be->set_option("GenerateWarnings", values().get_int("GenerateWarnings"));
-    _export_be->set_option("GenerateCreateIndex", values().get_int("GenerateCreateIndex"));
-    _export_be->set_option("NoUsersJustPrivileges", values().get_int("NoUsersJustPrivileges"));
-    _export_be->set_option("NoViewPlaceholders", values().get_int("NoViewPlaceholders"));
-    _export_be->set_option("GenerateInserts", values().get_int("GenerateInserts"));
-    _export_be->set_option("NoFKForInserts", values().get_int("NoFKForInserts"));
-    _export_be->set_option("TriggersAfterInserts", values().get_int("TriggersAfterInserts"));
-    _export_be->set_option("OmitSchemata", values().get_int("OmitSchemata"));
-    _export_be->set_option("GenerateUse", values().get_int("GenerateUse"));
+    _export_be->set_option("GenerateDrops", values().get_int("GenerateDrops") != 0);
+    _export_be->set_option("GenerateSchemaDrops", values().get_int("GenerateSchemaDrops") != 0);
+    _export_be->set_option("SkipForeignKeys", values().get_int("SkipForeignKeys") != 0);
+    _export_be->set_option("SkipFKIndexes", values().get_int("SkipFKIndexes") != 0);
+    _export_be->set_option("GenerateWarnings", values().get_int("GenerateWarnings") != 0);
+    _export_be->set_option("GenerateCreateIndex", values().get_int("GenerateCreateIndex") != 0);
+    _export_be->set_option("NoUsersJustPrivileges", values().get_int("NoUsersJustPrivileges") != 0);
+    _export_be->set_option("NoViewPlaceholders", values().get_int("NoViewPlaceholders") != 0);
+    _export_be->set_option("GenerateInserts", values().get_int("GenerateInserts") != 0);
+    _export_be->set_option("NoFKForInserts", values().get_int("NoFKForInserts") != 0);
+    _export_be->set_option("TriggersAfterInserts", values().get_int("TriggersAfterInserts") != 0);
+    _export_be->set_option("OmitSchemata", values().get_int("OmitSchemata") != 0);
+    _export_be->set_option("GenerateUse", values().get_int("GenerateUse") != 0);
 
     _export_be->set_option("TablesAreSelected", _table_filter->get_active());
     _export_be->set_option("TriggersAreSelected", _trigger_filter->get_active());
     _export_be->set_option("RoutinesAreSelected", _routine_filter->get_active());
     _export_be->set_option("ViewsAreSelected", _view_filter->get_active());
     _export_be->set_option("UsersAreSelected", _user_filter->get_active());
+
+    _export_be->set_option("OutputScriptHeader", header);
     
     // take target version from the version in the model
     _export_be->set_db_options_for_version(_export_be->get_catalog()->version());
