@@ -63,10 +63,10 @@ def handleLiveTreeContextMenu(name, sender, args):
         item.add_clicked_callback(lambda sender=sender, tables_selected=tables_selected: show_table_inspector(sender, tables_selected , 'indexes' if show_index_page else None))
         if from_schema_inspector:
             menu.insert_item(0, item)
+            menu.insert_item(1, mforms.newMenuItem("", mforms.SeparatorMenuItem))
         else:
             menu.insert_item(0 if show_index_page else 1, item)
-        menu.add_separator()
-
+            #menu.insert_item(1 if show_index_page else 2, mforms.newMenuItem("", mforms.SeparatorMenuItem))
 
 class TableInfoPanel(mforms.Box):
     caption = "Info"
@@ -217,9 +217,9 @@ class TableInfoPanel(mforms.Box):
         self.analyze_btn.add_clicked_callback(self.analyze_table)
         self.analyze_btn.enable_internal_padding(False)
         
-        table.add(mforms.newLabel("Information on this page may be outdated. Click the "), 0, 1, 0, 1, mforms.HFillFlag)
+        table.add(mforms.newLabel("Information on this page may be outdated. Click "), 0, 1, 0, 1, mforms.HFillFlag)
         table.add(self.analyze_btn,                                                     1, 2, 0, 1, mforms.HFillFlag)
-        table.add(mforms.newLabel(" button to update it."),                              2, 3, 0, 1, mforms.HFillFlag)
+        table.add(mforms.newLabel(" to update it."),                              2, 3, 0, 1, mforms.HFillFlag)
         
         bbox.add(table, False, False)
         
@@ -262,12 +262,15 @@ class TableInfoPanel(mforms.Box):
 #                     setattr(columnsVal, col.lower(), rset.stringFieldValueByName(col))
 
                 ok = rset.nextRow()
-            
-            self.table_size.set_text(human_size(int(column_values['data_length']) + int(column_values['index_length'])))
-            self.data_length.set_text(human_size(int(column_values['data_length'])))
-            self.index_length.set_text(human_size(int(column_values['index_length'])))
-            self.data_free.set_text(human_size(int(column_values['data_free'])))
-            self.max_data_length.set_text(human_size(int(column_values['max_data_length'])))
+
+            try:
+                self.table_size.set_text(human_size(int(column_values['data_length']) + int(column_values['index_length'])))
+                self.data_length.set_text(human_size(int(column_values['data_length'])))
+                self.index_length.set_text(human_size(int(column_values['index_length'])))
+                self.data_free.set_text(human_size(int(column_values['data_free'])))
+                self.max_data_length.set_text(human_size(int(column_values['max_data_length'])))
+            except Exception, e:
+                log_error("Error displaying table info for %s.%s: %s\n" % (self._schema, self._table, e))
 
         try:
             rset = self.editor.executeManagementQuery("select count(*) column_count from information_schema.columns WHERE table_schema = '%s' and table_name = '%s'" % (self._schema, self._table), 0)
@@ -524,16 +527,18 @@ class TableIndexInfoPanel(mforms.Box):
         self.index_list.add_column(mforms.StringColumnType, "Columns", 200, False)
         self.index_list.end_columns()
         self.index_list.add_changed_callback(self.index_selected)
-        self.index_list.set_size(500, -1)
+        self.index_list.set_size(450, -1)
         table.add(self.index_list, 0, 1, 1, 2, mforms.HFillFlag|mforms.VFillFlag)
 
-        table.add(make_title("Index Details"), 1, 2, 0, 1, mforms.HFillFlag|mforms.HExpandFlag)
-        
+        dhbox = mforms.newBox(True)
+        dhbox.add(make_title("Index Details"), False, True)
         self.drop_index = mforms.newButton()
         self.drop_index.set_text("Drop Index")
         self.drop_index.set_enabled(False)
         self.drop_index.add_clicked_callback(self.do_drop_index)
-        table.add(self.drop_index, 2, 3, 0, 1, mforms.HFillFlag)
+        dhbox.add_end(self.drop_index, False, True)
+        table.add(dhbox, 1, 3, 0, 1, mforms.HFillFlag|mforms.HExpandFlag)
+        
         self.info = mforms.newTable()
         table.add(self.info, 1, 3, 1, 2, mforms.HFillFlag|mforms.HExpandFlag|mforms.VFillFlag)
 
@@ -780,7 +785,6 @@ class TableTriggerManager(TableManDefs, TriggerManager):
            ]
     def __init__(self, editor):
         TriggerManager.__init__(self, editor, None)
-        self.inspect_btn.show(False)
         self.set_managed()
         self.set_release_on_add()
     
@@ -800,7 +804,6 @@ class TableColumnManager(TableManDefs, ColumnManager):
                ("COLUMN_COMMENT", StringColumnType, "Comments", 200, None)]
     def __init__(self, editor):
         ColumnManager.__init__(self, editor, None)
-        self.inspect_btn.show(False)
         self.set_managed()
         self.set_release_on_add()
     
@@ -823,7 +826,6 @@ class ConstraintManager(TableManDefs, ObjectManager):
                ("referenced_column_name", StringColumnType, "Referenced Column", 100, None)]
     def __init__(self, editor):
         ObjectManager.__init__(self, editor, None)
-        self.inspect_btn.show(False)
         self.set_managed()
         self.set_release_on_add()
     
@@ -850,12 +852,11 @@ class PartitionManager(TableManDefs, ObjectManager):
                ("data_length", IntegerColumnType, "Data length", 100, None),
                ("max_data_length", IntegerColumnType, "max data length", 100, None),
                ("index_length", IntegerColumnType, "Index length", 100, None),
-               ("create_time", StringColumnType, "Create time", 100, None),
-               ("update_time", StringColumnType, "Update time", 100, None),
-               ("check_time", StringColumnType, "Check time", 100, None)]
+               ("create_time", StringColumnType, "Create time", 150, None),
+               ("update_time", StringColumnType, "Update time", 150, None),
+               ("check_time", StringColumnType, "Check time", 150, None)]
     def __init__(self, editor):
         ObjectManager.__init__(self, editor, None)
-        self.inspect_btn.show(False)
         self.set_managed()
         self.set_release_on_add()
     
@@ -868,7 +869,6 @@ class GrantsTableManager(TableManDefs, ObjectManager):
     def __init__(self, editor):
         self.columns = []
         ObjectManager.__init__(self, editor, None)
-        self.inspect_btn.show(False)
        
         self.set_managed()
         self.set_release_on_add()
@@ -959,7 +959,6 @@ class GrantsColumnManager(TableManDefs, ObjectManager):
     def __init__(self, editor):
         self.columns = []
         ObjectManager.__init__(self, editor, None)
-        self.inspect_btn.show(False)       
         self.set_managed()
         self.set_release_on_add()
         
@@ -1008,9 +1007,7 @@ class GrantsColumnManager(TableManDefs, ObjectManager):
             else:
                 fields.append(field)
         
-        output.append("SELECT CONCAT(Db,'.',Table_name) as scope,%(sel_fields)s FROM mysql.tables_priv WHERE '%(schema)s' LIKE Db AND '%(table)s' LIKE Table_name" % {'sel_fields' : ",".join(fields), 'schema' : self.schema, 'table' : self.table})
-        
-        output.append("SELECT CONCAT(Db,'.',Table_name,'.',Column_name) as scope,%(sel_fields)s FROM mysql.columns_priv WHERE '%(schema)s' LIKE Db AND '%(table)s' LIKE Table_name" % {'sel_fields' : ",".join(fields), 'schema' : self.schema, 'table' : self.table})
+        output.append("SELECT CONCAT(Db,'.',Table_name,'.',Column_name) as scope,%(sel_fields)s FROM mysql.columns_priv WHERE Db LIKE '%(schema)s' AND Table_name LIKE '%(table)s'" % {'sel_fields' : ",".join(fields), 'schema' : self.schema, 'table' : self.table})
         return output
 
     def refresh(self):
